'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Immutable = require('immutable');
var ulid = require('ulid');

var _require = require('./queryset'),
    QuerySet = _require.QuerySet;

var _require2 = require('./manager'),
    createObjectManagerConstructor = _require2.createObjectManagerConstructor;

var _require3 = require('./emitter'),
    createChangesEmitter = _require3.createChangesEmitter;

/*
 * Default setup
 */

var PRIMARY_KEY_NAME = '_pk';

var Entry = function () {
  function Entry(_ref) {
    var manager = _ref.manager,
        values = _ref.values,
        setup = _ref.setup;
    (0, _classCallCheck3.default)(this, Entry);

    this.manager = manager;
    this.values = values;
    this.setup = setup;
  }

  (0, _createClass3.default)(Entry, [{
    key: 'assign',
    value: function assign() {
      var rawValues = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      Object.assign(this, rawValues);
    }
  }, {
    key: 'save',
    value: function save() {
      this.values = this.manager.sync(this.values);
      return this;
    }
  }, {
    key: 'update',
    value: function update() {
      var rawValues = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      this.assign(rawValues);
      return this.save();
    }
  }, {
    key: 'delete',
    value: function _delete() {
      this.values = this.manager.unsync(this.values);
      return this;
    }
  }, {
    key: 'toCleanValues',
    value: function toCleanValues() {
      return this.values.remove(this.setup.get('primaryKeyName'));
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {
      return this.toCleanValues().toJS();
    }
  }, {
    key: 'pk',
    get: function get() {
      var pkName = this.setup.get('primaryKeyName');
      return this.values.get(pkName);
    }
  }]);
  return Entry;
}();

var defaultSetup = {
  EntryConstructor: Entry,
  QuerySetConstructor: QuerySet,
  primaryKeyName: PRIMARY_KEY_NAME
};

/*
 * Composition
 */

var applyMixin = function applyMixin(_ref2) {
  var name = _ref2.name,
      setup = _ref2.setup,
      mixin = _ref2.mixin;
  return mixin({ Base: setup.get(name), setup: setup }, { setup: setup });
};

var createMixin = function createMixin(_ref3) {
  var name = _ref3.name,
      mixin = _ref3.mixin;
  return function (_ref4) {
    var setup = _ref4.setup;
    return setup.set(name, applyMixin({ name: name, setup: setup, mixin: mixin }));
  };
};

var composeSetup = function composeSetup(_ref5) {
  var model = _ref5.model,
      _ref5$setup = _ref5.setup,
      setup = _ref5$setup === undefined ? Immutable.Map(defaultSetup) : _ref5$setup,
      _ref5$composers = _ref5.composers,
      composers = _ref5$composers === undefined ? [] : _ref5$composers;

  var composer = Immutable.List(composers).get(0);

  return composer ? composeSetup({
    model: model,
    setup: composer({ model: model, setup: setup }),
    composers: Immutable.List(composers).shift(0)
  }) : setup;
};

/*
 * API
 */

var createEntryMixin = function createEntryMixin(mixin) {
  return createMixin({ name: 'EntryConstructor', mixin: mixin });
};
var createQuerySetMixin = function createQuerySetMixin(mixin) {
  return createMixin({ name: 'QuerySetConstructor', mixin: mixin });
};

var createModel = function createModel(composers) {
  var model = { id: ulid() };

  model.setup = composeSetup({
    model: model,
    setup: Immutable.Map(defaultSetup),
    composers: Array.isArray(composers) ? composers : [composers]
  });

  model.inject = function (composer) {
    model.setup = composer({ model: model, setup: model.setup });
    model.objects = model.objects.useSetup(model.setup);
  };

  var QuerySetConstructor = model.setup.get('QuerySetConstructor');
  var ObjectManagerConstructor = createObjectManagerConstructor(QuerySetConstructor);

  model.emitter = createChangesEmitter();
  model.objects = new ObjectManagerConstructor(model);

  return model;
};

var mixModelComposers = function mixModelComposers() {
  var composers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  return function (_ref6) {
    var setup = _ref6.setup;
    return composeSetup({ setup: setup, composers: composers });
  };
};

module.exports = {
  createEntryMixin: createEntryMixin,
  createQuerySetMixin: createQuerySetMixin,
  createModel: createModel,
  mixModelComposers: mixModelComposers
};
