'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _get2 = require('babel-runtime/helpers/get');

var _get3 = _interopRequireDefault(_get2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('./model'),
    createModel = _require.createModel,
    createEntryMixin = _require.createEntryMixin;

var _require2 = require('./field'),
    withField = _require2.withField;

var TARGET_PK_FIELD = 'targetPk';
var ROOT_PK_FIELD = 'rootPk';

/*
 * Generate relation data
 */

var assignRelation = function assignRelation(name, _ref) {
  var model = _ref.model,
      _ref$relatedName = _ref.relatedName,
      relatedName = _ref$relatedName === undefined ? null : _ref$relatedName,
      _ref$hasMany = _ref.hasMany,
      hasMany = _ref$hasMany === undefined ? false : _ref$hasMany;
  return { model: model, hasMany: hasMany, relatedName: relatedName };
};

var parseRelation = function parseRelation(name) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return opts.model ? assignRelation(name, opts) : assignRelation(name, { model: opts });
};

/*
 * Association
 */

var createAssociation = function createAssociation(_ref2) {
  var root = _ref2.root,
      target = _ref2.target;
  return {
    src: {
      model: root,
      pkField: ROOT_PK_FIELD
    },
    dest: {
      model: target,
      pkField: TARGET_PK_FIELD
    },
    model: createModel([withField(ROOT_PK_FIELD), withField(TARGET_PK_FIELD)])
  };
};

var reverseAssociation = function reverseAssociation(_ref3) {
  var src = _ref3.src,
      dest = _ref3.dest,
      model = _ref3.model;
  return { src: dest, dest: src, model: model };
};

/*
 * Many relation
 */

var createManyQuerysetConstructor = function createManyQuerysetConstructor(QuerySetConstructor) {
  return function (_QuerySetConstructor) {
    (0, _inherits3.default)(_class, _QuerySetConstructor);

    function _class(_ref4) {
      var association = _ref4.association,
          entry = _ref4.entry;
      (0, _classCallCheck3.default)(this, _class);

      var _this = (0, _possibleConstructorReturn3.default)(this, (_class.__proto__ || Object.getPrototypeOf(_class)).call(this, { parent: association.model.objects.filter((0, _defineProperty3.default)({}, association.src.pkField, entry.pk)) }));

      _this.association = association;
      _this.originEntry = entry;
      return _this;
    }

    (0, _createClass3.default)(_class, [{
      key: 'entries',
      value: _regenerator2.default.mark(function entries() {
        var _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, entry;

        return _regenerator2.default.wrap(function entries$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context.prev = 3;
                _iterator = (0, _get3.default)(_class.prototype.__proto__ || Object.getPrototypeOf(_class.prototype), 'entries', this).call(this)[Symbol.iterator]();

              case 5:
                if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                  _context.next = 12;
                  break;
                }

                entry = _step.value;
                _context.next = 9;
                return this.association.dest.model.objects.get({ pk: entry[this.association.dest.pkField] });

              case 9:
                _iteratorNormalCompletion = true;
                _context.next = 5;
                break;

              case 12:
                _context.next = 18;
                break;

              case 14:
                _context.prev = 14;
                _context.t0 = _context['catch'](3);
                _didIteratorError = true;
                _iteratorError = _context.t0;

              case 18:
                _context.prev = 18;
                _context.prev = 19;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 21:
                _context.prev = 21;

                if (!_didIteratorError) {
                  _context.next = 24;
                  break;
                }

                throw _iteratorError;

              case 24:
                return _context.finish(21);

              case 25:
                return _context.finish(18);

              case 26:
              case 'end':
                return _context.stop();
            }
          }
        }, entries, this, [[3, 14, 18, 26], [19,, 21, 25]]);
      })
    }, {
      key: 'add',
      value: function add(entry) {
        var _association$model$ob2;

        this.association.model.objects.createTransaction();

        this.association.model.objects.getOrCreate((_association$model$ob2 = {}, (0, _defineProperty3.default)(_association$model$ob2, this.association.src.pkField, this.originEntry.pk), (0, _defineProperty3.default)(_association$model$ob2, this.association.dest.pkField, entry.pk), _association$model$ob2));

        this.association.model.objects.commitTransaction();
      }
    }, {
      key: 'remove',
      value: function remove(entry) {
        var _association$model$ob3;

        this.association.model.objects.createTransaction();

        var associatedEntry = this.association.model.objects.get((_association$model$ob3 = {}, (0, _defineProperty3.default)(_association$model$ob3, this.association.src.pkField, this.originEntry.pk), (0, _defineProperty3.default)(_association$model$ob3, this.association.dest.pkField, entry.pk), _association$model$ob3));

        if (associatedEntry) associatedEntry.delete();

        this.association.model.objects.commitTransaction();
      }
    }]);
    return _class;
  }(QuerySetConstructor);
};

var createWithManyAccessor = function createWithManyAccessor(_ref5) {
  var fieldName = _ref5.fieldName,
      association = _ref5.association,
      setup = _ref5.setup;

  var QuerySetConstructor = createManyQuerysetConstructor(setup.get('QuerySetConstructor'));

  return createEntryMixin(function (_ref6) {
    var Base = _ref6.Base;
    return function (_Base) {
      (0, _inherits3.default)(_class2, _Base);

      function _class2() {
        (0, _classCallCheck3.default)(this, _class2);
        return (0, _possibleConstructorReturn3.default)(this, (_class2.__proto__ || Object.getPrototypeOf(_class2)).apply(this, arguments));
      }

      (0, _createClass3.default)(_class2, [{
        key: fieldName,
        get: function get() {
          return new QuerySetConstructor({ association: association, entry: this });
        }
      }]);
      return _class2;
    }(Base);
  });
};

/*
 * One relation
 */

var createWithOneAccessor = function createWithOneAccessor(_ref7) {
  var fieldName = _ref7.fieldName,
      association = _ref7.association;
  return createEntryMixin(function (_ref8) {
    var Base = _ref8.Base;
    return function (_Base2) {
      (0, _inherits3.default)(_class3, _Base2);

      function _class3() {
        (0, _classCallCheck3.default)(this, _class3);
        return (0, _possibleConstructorReturn3.default)(this, (_class3.__proto__ || Object.getPrototypeOf(_class3)).apply(this, arguments));
      }

      (0, _createClass3.default)(_class3, [{
        key: fieldName,
        get: function get() {
          var associatedEntry = association.model.objects.get((0, _defineProperty3.default)({}, association.src.pkField, this.pk));

          return associatedEntry ? association.dest.model.objects.get({
            pk: associatedEntry[association.dest.pkField]
          }) : null;
        },
        set: function set(entry) {
          association.model.objects.createTransaction();

          association.model.objects.getOrCreate((0, _defineProperty3.default)({}, association.src.pkField, this.pk)).update((0, _defineProperty3.default)({}, association.dest.pkField, entry.pk));

          association.model.objects.commitTransaction();
        }
      }]);
      return _class3;
    }(Base);
  });
};

/*
 * API
 */

var createRelationComposer = function createRelationComposer(name) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var rootAccessor = arguments[2];
  return function (_ref9) {
    var model = _ref9.model,
        setup = _ref9.setup;

    var relation = parseRelation(name, opts);
    var association = createAssociation({ root: model, target: relation.model });
    var relationOpts = {
      setup: setup,
      fieldName: relation.relatedName,
      association: reverseAssociation(association)
    };

    if (relation.relatedName && !relation.hasMany) {
      relation.model.inject(createWithOneAccessor(relationOpts));
    } else if (relation.relatedName && relation.hasMany) {
      relation.model.inject(createWithManyAccessor(relationOpts));
    }

    return rootAccessor({ association: association, setup: setup, fieldName: name })({ model: model, setup: setup });
  };
};

var withMany = function withMany(name) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return createRelationComposer(name, opts, createWithManyAccessor);
};
var withOne = function withOne(name) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return createRelationComposer(name, opts, createWithOneAccessor);
};

module.exports = {
  withOne: withOne,
  withMany: withMany
};
